@@ -1,6 +1,6 @@
 #!/usr/local/bin/perl -w
 
-# $Id: test.pl 2488 2006-02-07 22:24:43Z timbo $
+# $Id: test.pl 6734 2006-07-30 22:42:07Z timbo $
 #
 # Copyright (c) 1994-1998 Tim Bunce
 #
@@ -28,6 +28,11 @@ use Config;
 use Getopt::Long;
 use strict;
 
+our $has_devel_leak = eval {
+    local $^W = 0; # silence "Use of uninitialized value $DynaLoader::args[0] in subroutine entry";
+    require Devel::Leak;
+};
+
 $::opt_d = 0;
 $::opt_l = '';
 $::opt_h = 0;
@@ -35,7 +40,7 @@ $::opt_m = 0;		# basic memory leak test: "perl test.pl -m NullP"
 $::opt_t = 0;		# thread test
 $::opt_n = 0;		# counter for other options
 
-GetOptions(qw(d=i h=i l=s m t=i n=i))
+GetOptions(qw(d=i h=i l=s m=i t=i n=i))
     or die "Usage: $0 [-d n] [-h n] [-m] [-t n] [-n n] [drivername]\n";
 
 my $count = 0;
@@ -71,21 +76,22 @@ if (0) {
 
 if ($::opt_m) {
     #$dbh->trace(9);
-    my $level = 4;
+    my $level = $::opt_m;
     my $cnt = 10000;
     print "Using $driver, same dbh...\n";
-    #for (my $i=0; $i<$cnt; ++$i) { mem_test($dbh, undef, $level) }
+    for (my $i=0; $i<$cnt; ++$i) { mem_test($dbh, undef, $level, undef, undef, undef) }
     print "Using NullP, reconnecting each time...\n";
-    #for (my $i=0; $i<$cnt; ++$i) { mem_test(undef, ['dbi:NullP:'], $level) }
+    for (my $i=0; $i<$cnt; ++$i) { mem_test(undef, ['dbi:NullP:'], $level, undef, undef, undef) }
     print "Using ExampleP, reconnecting each time...\n";
-    mem_test(undef, ['dbi:ExampleP:'], $level) while 1;
-    #mem_test(undef, ['dbi:mysql:VC'], $level, "select * from campaigns where length(?)>0") while 1;
+    my $r_develleak = 0;
+    mem_test(undef, ['dbi:NullP:'], $level, undef, undef, \$r_develleak) while 1;
+    #mem_test(undef, ['dbi:mysql:VC'], $level, "select * from campaigns where length(?)>0", 0, undef) while 1;
 }
 elsif ($::opt_t) {
 	thread_test();
 }
 else {
-
+    
     # new experimental connect_test_perf method
     DBI->connect_test_perf("dbi:$driver:", '', '', {
 	dbi_loops=>5, dbi_par=>20, dbi_verb=>1
@@ -129,17 +135,39 @@ exit 0;
 
 
 sub mem_test {	# harness to help find basic leaks
-    my ($orig_dbh, $connect, $level, $select, $params) = @_;
+    my ($orig_dbh, $connect, $level, $select, $params, $r_develleak) = @_;
     $select ||= "select mode,ino,name from ?";
     $params ||= [ '.' ];
-    my $dbh = $orig_dbh || DBI->connect(@$connect);
+
+    # this can be used to force a 'leak' to check memory use reporting
+    #$main::leak .= " " x 1000;
     system("echo $count; $ps$$") if (($count++ % 500) == 0);
+
+    my $dbh = $orig_dbh || DBI->connect(@$connect);
+    $dbh->{RaiseError} = 1;
     my $cursor_a;
+
+    my ($dl_count, $dl_handle);
+    if ($$r_develleak++) {
+        $dbh->trace(2);
+        $dl_count = Devel::Leak::NoteSV($dl_handle);
+    }
+
     $cursor_a = $dbh->prepare($select)		if $level >= 2;
     $cursor_a->execute(@$params)		if $level >= 3;
+    $cursor_a->fetchrow_hashref()        	if $level >= 4;
     my $rows = $cursor_a->fetchall_arrayref({})	if $level >= 4;
     $cursor_a->finish if $cursor_a && $cursor_a->{Active};
+    undef $cursor_a;
+
+    @{$dbh->{ChildHandles}} = ();
+
+    die Devel::Leak::CheckSV($dl_handle)-$dl_count
+        if $dl_handle;
+
     $dbh->disconnect unless $orig_dbh;
+    undef $dbh;
+
 }
 
 
